<!-- K: navigate -->
<!-- S: Plan a route and determine your location. -->

Being able to navigate in the wilderness can help prevent you from getting lost or allow you to find your way back to safety.

## Be prepared
<!-- K: prepare, gear, knowledge, skill, map, compass, flashlight, gps -->
<!-- S: Bring a map, compass, and flashlight. Learn how to navigate with a map and compass and plan routes. -->

### Gear
**Necessities**:
- Map
- Compass
- Flashlight/headlamp

**Optional**:
- GPS device

### Knowledge and skills
Start by learning how to do the following:
- Navigate with a map and compass.
- Plan a route.
- Set declination on a compass.
- Determine your location on a map.
- Measure distances on a map.
- Read a topographic map.

Once you have mastered these skills, you should learn how to do the following:
- Use a GPS device for navigation.
- Utilize advanced navigation techniques, like using the sun and stars.
- Take a detour.

## Route planning
<!-- K: route, plan, trip, detour -->
<!-- S: Plan your route to avoid getting lost and ensure you reach your destination safely. -->

When planning a route, choose one that matches your skill and fitness levels using a topographic map to gauge terrain difficulty. Use the map to identify landmarks along the way to help you stay on track. Also be sure to identify water sources, alternative routes, and the direction to roads or towns.

## Movement
<!-- K: move, terrain, mountain, cold, desert, jungle, night, river, cross -->
<!-- S: Move carefully and adapt to the terrain. Avoid following animal trails. -->

Moving carefully and adapting to the terrain are important for preventing injury or getting lost. In general, keep your body weight balanced over your feet, watch where you step, and take frequent breaks to rest. Stay on the designated trails at all times and be cautious of animal trails; they may lead to resources like water but can also get you lost.

### Mountains

The steep slopes of mountainous terrain can tire you out or injure your joints, so be extra vigilant about taking breaks. To make it easier to follow uphill trails, walk in a zigzag path; this pattern will increase the distance you travel but it will be less steep. When descending, maintain a bend in your knees to protect your joints. If you are climbing a tall mountain, take your time so that you can acclimate to higher altitudes without getting sick.

### Snow

It can be taxing to travel in snowy conditions and you don't know for sure what you are stepping on; the snow may be hiding a patch of ice, a hole, or water. You can use a pole or stick to test ice and snow conditions before stepping. Avoid walking on frozen bodies of water unless you know the ice thickness. If you have snowshoes or skis, use them to help prevent your feet from sinking into the snow.

Watch out for areas prone to avalanches. They are usually marked.

### Desert

Deserts are typically hot and dry, with potentially cold nights, so be prepared by bringing appropriate clothing and extra water. Avoid traveling during the hottest part of the day and stick to areas of shade if possible. Due to the lack of visual features, paths in a desert may appear shorter than they actually are. If you are carrying electronic devices, be sure to protect them from extreme heat and direct sunlight.

### Jungle

Jungles feature dense vegetation, high humidity, and precipitation. Move with the vegetation rather than against it to conserve energy, and focus on distant landmarks rather than nearby objects to maintain your direction. The dense canopy makes it harder for search parties to find you and GPS devices may not be reliable. Always wear protective clothing to avoid insect bites and stings, and treat any wounds or scratches promptly to prevent infection.

### Night

If you can avoid traveling at night, then don't do it. If you have to travel, use a flashlight or headlamp to illuminate your path while moving slowly and carefully to avoid accidents. Use your hearing to identify helpful environmental cues like running water or distant roadways that can aid in orientation.

### Rivers and streams

When crossing rivers or streams, search for the narrowest section or areas where the water splits into multiple channels. Whenever possible, use bridges, fallen trees, or rocks as crossing points, but be cautious of slippery or loose surfaces. Avoid crossing where currents are strong or near ledges. If you have to wade through water to cross, then find another spot. Anytime you cross a river or stream, unbuckle your backpack and carry it in front of you to prevent being pulled underwater by its weight if you fall.

In emergency situations, following a river downstream may eventually lead to civilization, as many settlements are built near water sources.

### Detours

If there's an obstacle in your path, you may need to take a detour. To do this, find a feature to navigate to on the other side of the obstacle, then turn 90 degrees and walk to the end of the obstacle. Once there, turn back 90 degrees to your original heading and pass the obstacle. Finally, turn 90 degrees again and walk to the feature you picked out. Now you can resume following your original bearing.

## Trail markers
<!-- K: trail, blaze, marker, cairn, sign, pile, block, stack, paint, flag -->
<!-- S: Trail markers help you to stay on the trail. -->

Trail markers indicate where the trail is and distinguish one trail from another. Many ways are used to mark trails, but there is no universal standard.

### Blazes
Blazes are painted symbols on trees or rocks. Typically, they are rectangles, squares, circles, triangles, or crosses. In most places, each trail is painted with a different color and/or shape of blaze.

Some blazes may also indicate the trail's direction, but this may not apply everywhere.

![](file:///android_asset/survival_guide/trail_blazes.webp)

1. **Two blazes, top blaze to the left**: Continue to the left
2. **Two blazes, top blaze to the right**: Continue to the right
3. **One blaze**: Continue straight

While not always true, the color and/or shape of the blazes should match what appears on a map, except that white blazes are typically black on a map. If a trail is new, it may have small plastic strips or flags tied to a tree instead of a blaze.

### Cairns
Cairns are piles of rocks, typically used in places without large trees to blaze. Cairns are usually placed on the side of the trail, and you should be able to see multiple cairns at a time. In some places, cairns may be small stacks of stones (typically 3) and could be hard to locate.

### Signs
Trail signs provide information about which trail you are on, distance/direction to points of interest, or offer general information/alerts. Always read and abide by trail signs.

### Blockers
In some places, downed trees or debris are intentionally placed in front of a path to indicate it should not be used.

## GPS
<!-- K: gps, satellite, device, location, waypoint, beacon, battery, maps -->
<!-- S: A GPS device can help you determine your location and navigate to a specific point, but always have a backup plan in case the device fails. -->

A GPS device can help you determine your location and navigate to a specific point, but always have a backup plan in case the device fails.

### Location

A GPS uses satellites to determine your location. Ensure the device has a clear view of the sky so it can receive signals. Most GPS units display your coordinates in latitude and longitude, which can be plotted on a map.

### Waypoints

A waypoint is a specific location you want to navigate to. You can enter waypoints into your GPS to guide you to your destination. Most devices allow you to enter waypoints before your trip, but you can also add them as you go. Place waypoints frequently so you can backtrack if necessary.

If you use Trail Sense, you can use the Navigation, Beacons, Paths, and Photo Maps tools to assist with navigation.

### Battery

Your GPS device may run out of battery, so carry extra batteries or a portable charger to keep it powered.

### Digital maps

If your GPS supports digital maps, download maps of your area in advance to navigate even if you lose the signal.

## Compass
<!-- K: compass, bearing, declination, direction, north, south, east, west, orient, craft -->
<!-- S: A compass helps you determine direction. Be sure to set the same declination as your map. -->

A compass helps you determine direction and is essential for navigation, especially with a map. Do not use the compass around metal objects, as it will cause the reading to be inaccurate. If you are using a digital compass on a smartphone, it will need to be calibrated by rotating your device in a figure-8 pattern (the pattern may vary by device).

### Bearings

A bearing is the direction you must travel to reach a specific point. You can determine a bearing by aligning the compass needle with the desired direction on the compass housing. Once you have the bearing, follow it to your destination. Pick a prominent feature in the distance to navigate toward, avoiding the need for constant compass checks.

### Back bearings

A back bearing is the opposite direction of your current bearing. To calculate a back bearing: if your bearing is less than 180 degrees, add 180. If it's more than 180, subtract 180. On a compass, the back bearing is directly across from the bearing.

To use a back bearing to navigate, you need to take a bearing before hiking out. Keep this bearing written down or memorized, and as long as you didn't drastically change direction, following the back bearing will lead you close to your starting location. Every time you make a turn, you should record a bearing.

### Declination

Declination is the difference between true north and magnetic north. Adjust your compass for declination when navigating with a map. Some compasses have an adjustable declination ring; otherwise, you must calculate it manually. Most maps list declination in the legend and are oriented to true north.

## Map
<!-- K: map, location, scale, topographic, orientation, triangulate -->
<!-- S: You can use a map to navigate once you learn how to read it. Be sure it is up to date. -->

A map is a key tool for navigation, especially when paired with a compass.

### Orientation

You can orient a map to point north using a compass:
1. Place the compass on the map with the direction of travel arrow pointing toward the top of the map.
2. Rotate the map and compass together until the compass points north.

![](file:///android_asset/survival_guide/93.webp)

### Determining location

To determine your location on a map, identify prominent features around you and match them to the map. Use a compass to determine your direction and align the map accordingly.

To triangulate your location:
1. Orient the map to point north using your compass.
2. Identify and mark two prominent features around you on the map.
3. Measure the bearing to each feature using your compass.
4. Draw lines from each feature in the direction of the bearing on the map.
5. The point where the lines intersect is your location.

### Scale

Maps are drawn to scale, allowing you to determine distances between points. A bar on the map indicates the scale. You can measure distances using a ruler, string, or finger.

![](file:///android_asset/survival_guide/map_scale.webp)

### Topographic maps

Topographic maps show elevation changes and terrain features, making them helpful in navigating mountainous or hilly areas. Contour lines indicate elevation; the closer the lines are, the steeper the terrain. Elevation markers on contour lines help identify peaks and valleys.

![](file:///android_asset/survival_guide/contours.webp)

Typical map colors include:
- **Black**: Structures
- **Red**: Roads
- **Green**: Vegetation
- **White**: Lack of vegetation
- **Brown**: Contour lines
- **Blue**: Water

## Finding direction without a compass
<!-- K: sun, moon, star, shadow, watch, north, south, east, west, polaris, big dipper, cassiopeia, southern cross, constellation, advanced, technique -->
<!-- S: You can use the sun, moon, or stars to determine direction. If you need this, it's best to stay where you are and wait for help. -->

You can use the sun or stars to determine direction. If you need this, it's best to stay where you are and wait for help.

### Sun/Moon

In the early morning, the sun will be close to the east, and in the late afternoon, it will be close to the west. The same applies to the moon at moonrise and moonset. During the day, the sun will be in the southern half of the sky in the northern hemisphere, and in the northern half in the southern hemisphere.

### Shadows

If you measure which way shadows are moving over time, you can determine direction. This is because the Sun moves east to west throughout the sky, and therefore shadows move in the opposite direction.

1. Place a stick vertically in the ground and mark the shadow tip.
2. After 10-15 minutes, mark the new position of the shadow tip.
3. Draw a line between the two points for an east-west line. West will be in the direction of the first point.
4. Draw a line at a right angle to the east-west line. South (or north in the Southern Hemisphere) will point toward the sun.

![](file:///android_asset/survival_guide/direction_shadow.webp)

### Time

You can determine your direction using the position of the sun and the current time. This will be a rough approximation, but it's better than nothing. Point the hour hand of an analog watch at the sun; south is roughly halfway between the hour hand and 12 o'clock in the Northern Hemisphere. In the Southern Hemisphere, point the 12 o'clock position at the sun. If on daylight saving time, use 1 o'clock instead of 12.

![](file:///android_asset/survival_guide/direction_watch.webp)

### Stars

Being able to identify a handful of constellations will help you determine direction at night.

In the Northern Hemisphere, identify Polaris (the North Star) using the Big Dipper and Cassiopeia as reference points. Polaris is directly to the north.

![](file:///android_asset/survival_guide/northern_stars.webp)

In the Southern Hemisphere, you can use the Southern Cross to determine south. Extend a line from the top and bottom stars of the Cross's long axis, continuing about 4.5 times the distance between these stars toward the horizon to approximate the South Pole. This method is more difficult than the one used in the Northern Hemisphere.

![](file:///android_asset/survival_guide/southern_stars.webp)

## Craft: Compass
<!-- K: compass, needle, magnet, north, south, east, west, direction, craft -->
<!-- S: You can make a compass using a needle, knife, and a leaf. -->

You can craft a makeshift compass by magnetizing a needle and floating it in water.

![](file:///android_asset/survival_guide/bushcraft_compass.webp)

**Resources**
- Needle
- Knife
- Leaf or bottle cap
- Water

**Instructions**
1. Magnetize the needle by stroking it with the knife in one direction (to avoid dulling the blade, use the back of the knife). Stroke at least 50 times.
2. Place the needle on the leaf/cap and float it in water. The needle will slowly align to the north-south axis. Ensure no currents or wind could affect the needle.